



pip check
IF %ERRORLEVEL% NEQ 0 exit /B 1
jupyter labextension list
IF %ERRORLEVEL% NEQ 0 exit /B 1
jupyter server extension list
IF %ERRORLEVEL% NEQ 0 exit /B 1
jupyter serverextension list
IF %ERRORLEVEL% NEQ 0 exit /B 1
jupyter server extension list 1>server_extensions 2>&1
IF %ERRORLEVEL% NEQ 0 exit /B 1
cat server_extensions | grep -ie "jupyter_server_proxy.*OK"
IF %ERRORLEVEL% NEQ 0 exit /B 1
jupyter serverextension list 1>serverextensions 2>&1
IF %ERRORLEVEL% NEQ 0 exit /B 1
cat serverextensions | grep -ie "jupyter_server_proxy.*OK"
IF %ERRORLEVEL% NEQ 0 exit /B 1
jupyter labextension list 1>labextensions 2>&1
IF %ERRORLEVEL% NEQ 0 exit /B 1
cat labextensions | grep -ie "@jupyterlab/server-proxy.*3\.2\.2.*OK"
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
